$ErrorActionPreference = "Stop"
(Get-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py) | ForEach-Object { $_ -replace "try.+" , "" } | Set-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py
(Get-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py) | ForEach-Object { $_ -replace "from importlib import.+" , "" } | Set-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py;
(Get-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py) | ForEach-Object { $_ -replace "except ImportError.+" , "" } | Set-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py;
(Get-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py) | ForEach-Object { $_ -replace "import importlib_metadata.+" , "" } | Set-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py;
(Get-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py) | ForEach-Object { $_ -replace "__version__.+" , "__version__ = '3.2.0'" } | Set-Content C:\Python38\Lib\site-packages\jsonschema\__init__.py